#!/usr/bin/env python3
import time
import logging
import virgotools as vt
import PyFd as fd
import sys
import numpy as np
import os
from subprocess import Popen, PIPE

logger = logging.getLogger(__name__)

vt.log_to_cfg()

## input parsing ##
if len(sys.argv) not in (2, 3):
    sys.exit('Usage:Server config_file [cm_name]')
cfg_path = os.path.abspath(sys.argv[1])  # do this first, FdIO changes working dir
cfg_dir, cfg_file = os.path.split(cfg_path)

#Start the fdio server
fdio = vt.FdIO(sys.argv)

indir = vt.parse_key(cfg_path, 'FDIN_DIR')[0][0]
scope = vt.parse_key(cfg_path, 'SCOPE')[0][0]
dataset = vt.parse_key(cfg_path, 'DATASET')[0][0]
obs = vt.parse_key(cfg_path, 'OBSERVATORY')[0][0]
tag = vt.parse_key(cfg_path, 'TAG')[0][0]
start_gps = int(vt.parse_key(cfg_path, 'START_GPS')[0][0])
frame_len = int(vt.parse_key(cfg_path, 'FRAME_LEN')[0][0])

#Get the first filename
filename = f"{obs}-{tag}-{start_gps}-{frame_len}.gwf"

#Mark the state as configured
fd.CfgReachState(fd.CfgServerConfigured)

#Download the fist file
logger.info("Downloading the {filename} file from rucio")
p = Popen(["rucio","--verbose","download",f"{scope}:{filename}","--no-subdir",'--dir',f"{indir}"],stdout=PIPE,stderr=PIPE)

#Get the output
output, error = p.communicate()
print(output.decode())

if p.returncode!=0:
    logger.error(f"ERROR: The downloading of {filename} from the datalake failed, exiting")
    print("rucio download failed %d %s" % (p.returncode,error.decode() )) 
    raise SystemExit()

logger.info("Successfully downloaded the first file from Rucio")

#Mark the state as active
fd.CfgReachState(fd.CfgServerActive)

#Mark the state as golden
fd.CfgReachState(fd.CfgServerGolden)
next_start_gps = start_gps
prev_filename = ""

while not fd.CfgFinished():

    #Get the next frame
    in_frame = fdio.get_frame()

    #Check if we need to download the next file.
    if in_frame.gps>next_start_gps:
        next_start_gps += frame_len
        next_filename = f"{obs}-{tag}-{next_start_gps}-{frame_len}.gwf"

        logger.info(f"Downloading file from rucio with start gps = {next_start_gps}")
        p = Popen(["rucio","--verbose","download",f"{scope}:{next_filename}","--no-subdir",'--dir',f"{indir}"],stdout=PIPE,stderr=PIPE)

        #Delete the previous file
        if prev_filename!="":
            prev_filename_path = os.path.join(indir,prev_filename)
            logger.info(f"Removing {prev_filename_path}")
            os.remove(prev_filename_path)

        prev_filename = filename
        filename = next_filename

    #Check if the downloading process failed
    if p.poll() is not None:
        if p.returncode!=0:
            logger.error(f"ERROR: The downloading of {filename} from the datalake failed, exiting")
            #print("rucio download failed %d %s" % (p.returncode,p.stderr.read().decode() )) 
            raise SystemExit()

    if(fdio._fdio.contents.userInfo==None):
        fd.CfgMsgAddUserInfo("GPS: %i" %(in_frame.gps))
        fd.CfgMsgSendWithTimeout(0.001)  

    #Put the frame
    fdio.put_frame(in_frame)

    